import profig


class Config:
    def __init__(self, conf_type="dev"):
        """
        Класс конфигураций.
        В корне проекта требуются файлы config.cfg и configDev.cfg
        :param conf_type: Тип конфигураций (prod | dev)
        """
        self.conf_type = conf_type
        if conf_type == "prod":
            self.config = profig.Config('config.cfg')
        else:
            self.config = profig.Config('configDev.cfg')

        self.sync()

    def init(self, key, val):
        """
        Инициализация параметров конфигураций
        :param key: Ключ конфига
        :param val: Дефолтное значение конфига. Будет перезаписан
        :return:
        """
        self.config.init(key, val)

    def sync(self):
        """
        Метод синхронизирует конфиги и перезапишет дефолтные конфиги указанные через метод init()
        :return:
        """
        self.config.sync()

    def get(self, key):
        """
        Метод возвращает значение конфига
        :param key: Ключ конфига который надо вернуть
        :return: Значение конфига
        """
        try:
            conf_value = self.config[key]
            return conf_value
        except Exception as err:
            raise FileNotFoundError(f'Cannot find config: Key: {key}.  Error: {str(err)}')



import traceback


def handle():
    """
    Возвращает текст ошибки без ковычек, чтобы можно было записывать в  БД
    :return: Текст ошибки
    """
    formatted_lines = traceback.format_exc().splitlines()
    return f'{formatted_lines[-1]}; {formatted_lines[-2].replace(" ", "")}; {formatted_lines[-3].replace(" ", "", 2)}'


def error_handler(func):
    def func_wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as e:
            formatted_lines = traceback.format_exc().splitlines()
            text = f'Error: {formatted_lines[-1]}; Method: {formatted_lines[-2].replace(" ", "")}; ' \
                   f'{formatted_lines[-3].replace(" ", "", 2)}'
            print("Error!", text)
            return None
    return func_wrapper


# @error_handler
# def hello(a, b):
#     print(a/b)
#
#
# hello(5, 0)



from kafka import KafkaConsumer
import json
# Docs: https://kafka-python.readthedocs.io/en/master/apidoc/KafkaConsumer.html


class Consumer:
    def __init__(self, topics, group_id, brokers, msg_offset='earliest'):
        """
        Класс для прослушивания кафки
        :param topics: Прослушиваемый топик
        :param group_id: Группа слушателей
        :param brokers: Массив(list) из брокеров
        :param log: Функция логирования
        """
        self.topics = topics
        self.group_id = group_id
        self.brokers = brokers
        self.msg_offset = msg_offset
        self.consumer = None

        self.connect()

    def connect(self):
        """
        Создает подключение к кафке
        """
        try:
            self.consumer = KafkaConsumer(self.topics,
                                          group_id=self.group_id,
                                          bootstrap_servers=self.brokers,
                                          auto_offset_reset=self.msg_offset,
                                          # value_deserializer=lambda m: json.loads(m.decode('utf-8'))
                                          )
        except BaseException as err:
            raise err
        else:
            print('Consumer created sucessfully!')

    def listen(self, processor):
        """
        Начинает слушать кафку
        :param processor: функция обработки для каждого сообщения из кафки
        :return:
        """
        for message in self.consumer:
            processor(message)

    def close(self):
        print('Consumer closed')
        self.consumer.close()



from datetime import datetime
import json


def get_datetime_str_from_timestamp(timestamp):
    dt_object = datetime.fromtimestamp(timestamp)
    return dt_object.strftime("%Y-%m-%dT%H:%M:%S")


def get_datetime_str_from_now():
    now = datetime.now()
    return now.strftime("%Y-%m-%dT%H:%M:%S")


def get_datetime_str_from_str(str_val):
    return datetime.strptime(str_val, '%Y-%m-%d %H:%M:%S').strftime("%Y-%m-%dT%H:%M:%S")


def get_date_from_str(str_val):
    return datetime.strptime(str_val, '%Y-%m-%d %H:%M:%S')


def get_timestamp_from_date(date_val):
    return datetime.timestamp(date_val)


def nvl(val, rpl=""):
    if val:
        return val
    else:
        return rpl


def escape(str_val):
    return str_val.replace("\'", "").replace("\"", "")


def marshall(message):
    message = json.loads(message)
    return message


def unmarshall(message):
    msg = json.dumps(message)
    return msg






# http://ozzmaker.com/add-colour-to-text-in-python/

def gray(msg):
    print("\033[1;30m" + msg + "\033[0m")


def red(msg):
    print("\033[1;31m" + msg + "\033[0m")


def green(msg):
    print("\033[1;32m" + msg + "\033[0m")


def yellow(msg):
    print("\033[1;33m" + msg + "\033[0m")


def blue(msg):
    print("\033[1;34m" + msg + "\033[0m")


def magenta(msg):
    print("\033[1;35m" + msg + "\033[0m")


def cyan(msg):
    print("\033[1;36m" + msg + "\033[0m")


WHITE = "\033[1;30m{}\033[0m"
RED = "\033[1;31m{}\033[0m"
GREEN = "\033[1;32m{}\033[0m"
YELLOW = "\033[1;33m{}\033[0m"
BLUE = "\033[1;34m{}\033[0m"
MAGENTA = "\033[1;35m{}\033[0m"
CYAN = "\033[1;36m{}\033[0m"


def pr(*args, c=None, sep=' ', end='\n'):
    """
    w, r, g, y, b, m, c
    """

    msg = ""
    cnt = 0
    for i in args:
        cnt += 1
        if c is None:
            msg += str(i) + sep
        else:
            color = get_color_from_str(c, cnt)
            if color == 'w':
                msg += WHITE.format(i) + sep
            elif color == 'r':
                msg += RED.format(i) + sep
            elif color == 'g':
                msg += GREEN.format(i) + sep
            elif color == 'y':
                msg += YELLOW.format(i) + sep
            elif color == 'b':
                msg += BLUE.format(i) + sep
            elif color == 'm':
                msg += MAGENTA.format(i) + sep
            elif color == 'c':
                msg += CYAN.format(i) + sep
            else:
                msg += str(i) + sep
    msg += end
    print(msg, sep='', end='')


def get_color_from_str(p_str, pos):
    tmp_str = p_str.replace(" ", '').replace(",", '')
    if len(tmp_str) < pos:
        False
    else:
        return tmp_str[pos-1:pos]


# gray('12345')
# red('12345')
# green('12345')
# yellow('12345')
# blue('12345')
# magenta('12345')
# cyan('12345')
# pr("test", 1, True, "test2", "test3", "test4", "test5", "test6", c='r,w,b,g,y,m,c')



import json

'''
| 1  | TRANSACTION_NAME       | Transaction type is profile created               | String     | Yes |
| 2  | EVENT_ID               | Transaction ID                                    | Number     | Yes |
| 3  | MSISDN                 | MSISDN                                            | String     | Yes |
| 4  | EVENT_TIMESTAMP        | Date and time of the event as created in DMP      | Datetime   | Yes |
| 5  | SOURCE_EVENT_TIMESTAMP | Date and time of the event in the source system   | Datetime   | Yes |
| 6  | SOURCE                 | The source of event                               | String     | Yes |
| 7  | SUBS_KEY/BAN           | The customer number in DWH/DMP                    | Number     | No  |
| 8  | VEON_ID                | The customer id in VEON                           | String(24) | No  |
| 9  | P1                     | Free number parameter                             | Integer    | No  |
| 10 | P2                     | Free numeric parameters                           | Integer    | No  |
| 11 | P3                     | Free string parameter                             | String     | No  |
| 12 | P4                     | Free string parameter                             | String     | No  |
| 13 | P5                     | Free string parameter                             | String     | No  |
| 14 | P6                     | Free string parameter                             | String     | No  |
| 15 | P7                     | Free datetime parameter                           | Datetime   | No  |
| 16 | P8                     | Free numeric parameters                           | numeric    | No  |
| 17 | P9                     | Free numeric parameters                           | numeric    | No  |
| 18 | P10                    | Free datetime parameters                          | Datetime   | No  |
| 19 | Main Balance           | Free numeric parameters                           | numeric    | No  |
| 20 | B1                     | Free numeric parameters                           | numeric    | No  |
| 21 | B2                     | Free numeric parameters                           | numeric    | No  |
| 22 | B3                     | Free numeric parameters                           | numeric    | No  |
| 23 | B4                     | Free numeric parameters                           | numeric    | No  |


{
    "MSISDN": "xxxxxxxxx",
    "TRANSACTION_NAME": "TRIGGER_NAME",
    "EVENT_ID": "1234567890",
    "EVENT_TIMESTAMP": "2018-09-24T13:54:45",
    "SOURCE": "SAMPLE_DATA",
    "SOURCE_EVENT_TIMESTAMP": "2018-09-18T13:30:40",
    "BAN": "xxxxxxxxxxxxx",
    "VEON_ID": "xxxxxxxx",
    "P1": 0,
    "P2": 0,
    "P3": "",
    "P4": "",
    "P5": "",
    "P6": "",
    "P7": "2019-03-07T13:00:00",
    "P8": 0.0,
    "P9": 0.0,
    "P10": "2019-03-10T13:00:00",
    "MAIN_BALANCE": 0.0,
    "B1": 0.0,
    "B2": 0.0,
    "B3": 0.0,
    "B4": 0.0
}

'''


# "SUBS_KEY/BAN": "",  - will be created when create new instance of Responce
class Response:
    def __init__(self, transaction_name=None, event_id=None, msisdn=None, event_timestamp=None,
                 source_event_timestamp=None, source=None, resp_type='ban', resp_val=0):

        if transaction_name is None:
            raise AttributeError("TRANSACTION_NAME are not filled")
        if event_id is None:
            raise AttributeError("EVENT_ID are not filled")
        if msisdn is None:
            raise AttributeError("MSISDN are not filled")
        if event_timestamp is None:
            raise AttributeError("EVENT_TIMESTAMP are not filled")
        if source_event_timestamp is None:
            raise AttributeError("SOURCE_EVENT_TIMESTAMP are not filled")
        if source is None:
            raise AttributeError("SOURCE are not filled")

        # Написано так для сохранения последовательности из примера
        self.responce = {}
        self.responce['TRANSACTION_NAME'] = transaction_name
        self.responce['EVENT_ID'] = event_id
        self.responce['MSISDN'] = msisdn
        self.responce['EVENT_TIMESTAMP'] = event_timestamp
        self.responce['SOURCE_EVENT_TIMESTAMP'] = source_event_timestamp
        self.responce['SOURCE'] = source
        if resp_type == 'ban':
            self.responce['BAN'] = resp_val
        else:
            self.responce['SUBS_KEY'] = resp_val
        self.responce['VEON_ID'] = ""
        self.responce['P1'] = 0
        self.responce['P2'] = 0
        self.responce['P3'] = ""
        self.responce['P4'] = ""
        self.responce['P5'] = ""
        self.responce['P6'] = ""
        self.responce['P7'] = ""  # date
        self.responce['P8'] = 0.0
        self.responce['P9'] = 0.0
        self.responce['P10'] = ""  # date
        self.responce['MAIN_BALANCE'] = 0.0
        self.responce['B1'] = 0.0
        self.responce['B2'] = 0.0
        self.responce['B3'] = 0.0
        self.responce['B4'] = 0.0

    def set_param(self, key, val):
        self.responce[key] = val

    def set(self, key, val):
        try:
            if key not in ('SOURCE', 'BAN', 'SUBS_KEY', 'VEON_ID', 'P1', 'P2', 'P3', 'P4', 'P5',
                           'P6', 'P7', 'P8', 'P9', 'P10', 'MAIN_BALANCE', 'B1', 'B2', 'B3', 'B4'):
                raise NameError("Incorrect parameter for response")

            # String 11
            if key in ('SOURCE', 'SUBS_KEY', 'BAN', 'VEON_ID', 'P3', 'P4', 'P5', 'P6'):
                self.responce[key] = self.nvl(val, '')

            # Integer
            elif key in ('P1', 'P2'):
                self.responce[key] = int(self.nvl(val, 0))

            # Float 7
            elif key in ('P8', 'P9', 'MAIN_BALANCE', 'B1', 'B2', 'B3', 'B4'):
                self.responce[key] = float(self.nvl(val, 0.0))

            # Datetime 4
            elif key in ('P7', 'P10'):
                self.responce[key] = self.nvl(val, '')

        except TypeError as t_err:
            raise TypeError(f'Incorrect type in field. Key: {key} value: {val}.Err: {str(t_err)}')
        except BaseException as b_err:
            raise Exception(f'Cannot set value on response. Key: {key} value: {val}.Err: {str(b_err)}')

    def get_response_obj(self):
        return self.responce

    def get_response_str(self):
        str_res = json.dumps(self.responce, ensure_ascii=False)
        return str_res

    @staticmethod
    def nvl(msg, replace):
        if msg is None:
            return replace
        return msg


from Nika import config, postgres, logger, consumer, producer, error, util, colorprint as p
import sys, os


class Nika:
    def __init__(self, config_type='dev'):
        p.pr("################################################################################################", c='b')
        p.pr("##", f'Nika Started with config', config_type, c='b,c,y')
        self.config_type = config_type
        self.conf = self.conf_init()
        self.pg = self.postgres_init()
        self.log = self.log_init()
        self.srcData = None
        self.trgData = None

        self.console_info()

        self.log('I', 'Program started!')

    def conf_init(self):
        """
        Инициализация конфигов
        """
        try:
            return config.Config(self.config_type)
        except BaseException:
            text = error.handle()
            self.log('E', text)
            # sys.exit(0)
            os._exit(1)
        pass

    def postgres_init(self):
        try:
            user = self.conf.get('Postgres.user')
            password = self.conf.get('Postgres.password')
            host = self.conf.get('Postgres.host')
            port = self.conf.get('Postgres.port')
            database = self.conf.get('Postgres.database')
            return postgres.PostgresConn(user, password, host, port, database)
        except FileNotFoundError:
            text = error.handle()
            self.log('E', text)
            # sys.exit(0)
            os._exit(1)
        except BaseException:
            text = error.handle()
            self.log('E', text)
            # sys.exit(0)
            os._exit(1)

    def log_init(self):
        """
        Инициализация инстанса логирования.
        """
        try:
            app_name = self.conf.get('Nika.name')
            log_level = self.conf.get('Logger.level')
            lg = logger.Logger(app_name, self.config_type, log_level, self.pg)
            return lg.log
        except FileNotFoundError:
            text = error.handle()
            self.log('E', text)
            # sys.exit(0)
            os._exit(1)
        except BaseException:
            text = error.handle()
            self.log('E', text)
            # sys.exit(0)
            os._exit(1)

    def console_info(self):
        """
        Выводит информация о приложении в консоль.
        """
        if self.conf.get('Nika.srcData') == "kafka":
            p.pr("##", f'Source data is: "Kafka". Topic: {self.conf.get("kafkaConsumer.topic")}. '
                 f'Brokers: {self.conf.get("kafkaConsumer.brokers")}', c='b,c')

        if self.conf.get("Nika.trgData") == "kafka":
            p.pr('##', f'Target data is: "Kafka". Topic: "{self.conf.get("kafkaProducer.topic")}". '
                 f'Brokers: {self.conf.get("kafkaProducer.brokers")}', c='b,c')
        p.pr("################################################################################################", c='b')

    def init_trg_connection(self):
        if self.conf.get("Nika.trgData") == "kafka":
            topic = self.conf.get("kafkaProducer.topic")
            brokers = self.conf.get("kafkaProducer.brokers").replace(' ', '').replace('\"', '').split(',')
            self.trgData = producer.Producer(topic, brokers)
            if not self.trgData.connect():
                raise ConnectionError('Nika.trgData cannot connect to Kafka')
        else:
            raise ConnectionError(f'Nika.trgData incorrect or not found in config file. '
                                  f'Nika.trgData: {self.conf.get("Nika.trgData")}')

    def validate_data(self, message):
        return message

    def pre_process_data(self, message):
        return message

    def process_data(self, message):
        return message

    def post_process_data(self, message):
        return message

    def prepare_to_send(self, message):
        return message

    def send_data(self, message):
        if type(message) == list:
            for i in message:
                message_for_send = i
                if type(i) != str:
                    message_for_send = util.unmarshall(i)
                self.trgData.send(message_for_send)
        elif type(message) == dict:
            self.trgData.send(util.unmarshall(message))
        elif type(message) == str:
            self.trgData.send(message)
        else:
            raise TypeError(f'Incorrect type of message to send. Type: {type(message)}')

    def process(self, message):
        msg = message.value
        msg = self.validate_data(msg)
        msg = self.pre_process_data(msg)
        msg = self.process_data(msg)
        msg = self.post_process_data(msg)
        msg = self.prepare_to_send(msg)
        try:
            self.send_data(msg)
        except TypeError:
            text = error.handle()
            self.log('E', text)
            os._exit(1)
        except BaseException:
            text = error.handle()
            self.log('E', text)
            os._exit(1)

    def start(self):
        """
        :return:
        """
        try:
            self.init_trg_connection()
        except BaseException:
            text = error.handle()
            self.log('E', text)
            os._exit(1)

        if self.conf.get("Nika.srcData") == 'kafka':
            try:
                topics = self.conf.get("kafkaConsumer.topic")
                group_id = self.conf.get("kafkaConsumer.groupID")
                brokers = self.conf.get("kafkaConsumer.brokers").replace(' ', '').replace('\"', '').split(',')
                msg_offset = 'earliest'
                self.srcData = consumer.Consumer(topics, group_id, brokers, msg_offset)
                self.srcData.listen(self.process)
            except FileNotFoundError:
                text = error.handle()
                self.log('E', text)
            except BaseException:
                text = error.handle()
                self.log('E', text)
            finally:
                # Закрываем соединение с кафкой и коммитим сообщение,
                # чтобы не получить исключение обратно от этого сообщения
                if self.srcData:
                    self.srcData.close()
                os._exit(1)
        else:
            text = "Nika.srcData incorrect or not found in config file"
            self.log('E', text)
            os._exit(1)



import logging
from datetime import datetime
import os
from Nika import util, colorprint as p


class Logger:
    def __init__(self, app_name, conf_type, log_level, pg):
        """
        Создает логгер.
        :param app_name: Имя приложения
        :param log_level: Уровень логирования
        :param pg: Инстанс подключения к PostgreSql
        """
        self.app_name = app_name
        self.conf_type = conf_type
        self.log_level = log_level
        self.logger = logging.getLogger(self.app_name)
        self.logger.setLevel(logging.INFO)
        self.postgres = pg
        self.path_to_log = os.path.join(os.getcwd(), 'logs')
        self.log_file_name = None
        self.fh = None

        if not os.path.exists(self.path_to_log):
            os.makedirs(self.path_to_log)
            p.pr("##", 'Folder for logs created!', c='b,c')

        self.check_log_folder()

    def log(self, log_type, msg):
        """
        Метод для записи логов
        :param log_type: Тип лога: D, I, W, E, C
        :param msg: Текст лога
        """
        self.check_log_folder()
        message = msg
        if log_type == "D":
            if self.check_log_level("debug"):
                self.logger.debug(msg)

        elif log_type == "I":
            if self.check_log_level("info"):
                self.logger.info(msg)

        elif log_type == "W":
            if self.check_log_level("warning"):
                self.logger.warning(msg)

        elif log_type == "E":
            if self.check_log_level("error"):
                self.logger.error(msg)
                p.red(msg)
                if self.conf_type != 'dev':
                    self.postgres.new_log(self.app_name, "E", util.escape(message))

        elif log_type == "C":
            if self.check_log_level("critical"):
                self.logger.critical(msg)

    def check_log_level(self, level):
        """
        Проверяет тип лога из конфигов и входящих логов
        :param level: Уровень лога
        """
        if self.log_level == "debug":
            return True
        if self.log_level == "info" and level in ("info", "warning", "error", "critical"):
            return True
        if self.log_level == "warning" and level in ("warning", "error", "critical"):
            return True
        if self.log_level == "error" and level in ("error", "critical"):
            return True
        if self.log_level == "critical" and level == "critical":
            return True
        return False

    def check_log_folder(self):
        """
        Проверяет наличие папки по дате, при необходимости создает папку
        """
        name = datetime.today().strftime('%Y-%m-%d')
        # if not os.path.isfile(self.path_to_log + "/" + name + ".log"):
        if self.log_file_name is None:
            self.fh = logging.FileHandler(self.path_to_log + "/" + name + ".log", encoding="UTF-8")
            formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
            self.fh.setFormatter(formatter)
            self.logger.addHandler(self.fh)

            self.log_file_name = name

        if self.log_file_name != name:
            # Create the logging file, if file is not found
            self.fh = logging.FileHandler(self.path_to_log + "/" + name + ".log", encoding="UTF-8")
            formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
            self.fh.setFormatter(formatter)
            self.logger.addHandler(self.fh)

            self.log_file_name = name



from kafka import KafkaProducer
from kafka.errors import KafkaError


class Producer:
    def __init__(self, topics, brokers):
        """
        Конструктор класса Зкщвгсук
        :param topics: Топик для отправки сообщения
        :param brokers: Брокеры кафки
        """
        self.topics = topics
        self.brokers = brokers
        self.producer = None

    def connect(self):
        """
        Создает соединение к кафке
        :return: True при успешном соединении, False при неудачном
        """
        try:
            self.producer = KafkaProducer(bootstrap_servers=self.brokers)
            return True
        except KafkaError:
            print("Kafka Producer connection Error!", KafkaError)
            return False

    def send(self, message):
        """
        Метод отправки сообщения в кафку
        :param message: Сообщение для отправки
        :return: True при успешной отправке, False при неудачном
        """
        future = self.producer.send(self.topics, bytes(message, 'utf-8'))

        # Block for 'synchronous' sends
        try:
            record_metadata = future.get(timeout=10)
        except KafkaError as kafkaErr:
            # Decide what to do if produce request failed...
            print(kafkaErr)
            return False

        # Successful result returns assigned partition and offset
        # print(record_metadata.topic)
        # print(record_metadata.partition)
        # print(record_metadata.offset)
        if record_metadata.partition:
            return True
        else:
            return False



import setuptools

print(setuptools.find_packages())

with open("README.md", encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name="Nika",
    version="1.0",
    author="Bekkazy Kubanychbekov",
    author_email="bekkazy.k@gmail.com",
    description="Package for process data from some source",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/bekkazy-k/Nika",
    packages=setuptools.find_packages(),
    install_requires=[
        'kafka-python==1.4.6',
        'profig==0.4.1',
        'psycopg2==2.8.2'
    ],
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
)



# https://pynative.com/psycopg2-python-postgresql-connection-pooling/
# https://toster.ru/q/638535 - Реализовать по данному примеру
import psycopg2
from psycopg2 import pool
import datetime
from Nika import colorprint as p


class PostgresConn:
    def __init__(self, user, password, host, port, database):
        self.__pool = None
        self.user = user
        self.password = password
        self.host = host
        self.port = port
        self.database = database

        self.connect()

    def connect(self):
        try:
            self.__pool = psycopg2.pool.SimpleConnectionPool(1, 20, user=self.user, password=self.password,
                                                             host=self.host, port=self.port, database=self.database)
            if self.__pool:
                p.pr('##', 'Postgres connection pool created successfully', c='b,c')
            else:
                raise ConnectionRefusedError("error trying to open connection pool")
        except (Exception, psycopg2.DatabaseError) as error:
            raise ConnectionError(f'An error while trying to establish a connection with Postgres. '
                                  f'Error text: {str(error)}')

    def get_conn(self):
        if self.__pool:
            return self.__pool.getconn()
        else:
            raise ConnectionRefusedError("Connection pool is empty")

    def close_conn(self, connection):
        if self.__pool:
            self.__pool.putconn(connection)
        else:
            raise ConnectionRefusedError("Connection pool is empty")

    def disconnect(self):
        if self.__pool:
            self.__pool.closeall
        print("PostgreSQL connection pool is closed")

    def execute(self):
        # connection = self.get_conn()
        # cursor = connection.cursor()
        # cursor.execute("select * FROM vbiuser.mobile order by 1 asc")
        # mobile_records = cursor.fetchmany(size=100)
        #
        # for row in mobile_records:
        #     print(row)
        #
        # cursor.close()
        #
        # self.close_conn(connection)
        pass

    def insert(self, table, cols: list, rows: list):
        connection = None
        try:
            query_cols = ""
            query_vals = ""
            for i in cols:
                query_cols += i+", "
                query_vals += "%s,"
            query = f'INSERT INTO {table} ({query_cols[:-2]}) VALUES ({query_vals[:-1]})'

            connection = self.get_conn()
            if connection:
                cursor = connection.cursor()
                cursor.execute(query, rows)
                connection.commit()
                # count = cursor.rowcount
                # print(count, f'Record inserted successfully into {table} table')
            self.close_conn(connection)

        except (Exception, psycopg2.Error) as error:
            if connection:
                print(f'Failed to insert record into mobile table', error)
                raise ConnectionError(f'Failed to insert record into mobile table. Error text: {str(error)}')
            else:
                raise ConnectionError(f'Unknown error: {str(error)}')

    def new_log(self, src_project_name, log_type, log_text):
        self.insert('vbiuser.logs', ['created_at', 'updated_at', 'source', 'log_type', 'log_text'],
                    [datetime.datetime.now(), datetime.datetime.now(), src_project_name, log_type, log_text])



